#include "stepper.h"
#include "pin.h"
#include <Arduino.h>

namespace Stepper{

  int target = 0;
  int position = 0;

  int speed = 800; // step/s
  int mode = 4;

  unsigned long period = 1000000 / speed / mode;
  unsigned long tlastStep = 0;

  bool dir = false;
  bool enabled = false;

  void init(){
    pinMode(Pin::DIR, OUTPUT);
    pinMode(Pin::ENA, OUTPUT);
    pinMode(Pin::STEP, OUTPUT);

    setDirection(true);
    setTarget(0);
    enable(true);
  }

  void update(){
    if(target - position != 0){
      if((target > position) != dir){
        setDirection(!dir);
      } 
      if(micros() - tlastStep >= period){
        step();
      }
    }
  }

  void setDirection(bool ndir){
    dir = ndir;
    digitalWrite(Pin::DIR, dir);
  }

  void setTarget(int ntarget){
    target = ntarget*35;
  }

  void step(){
    digitalWrite(Pin::STEP,HIGH); 
    delayMicroseconds(8); 
    digitalWrite(Pin::STEP,LOW); 
    delayMicroseconds(8); 
    position += dir ? 1 : -1;
    tlastStep = micros();
  }
  
  void enable(bool state){
    enabled = state;
    digitalWrite(Pin::ENA, !state);
  }

  void disable(){
    enable(false);
  }

  bool isEnabled(){
    return enabled;
  }

} // namespace Stepper
